@@ -624,8 +624,9 @@ def load_dataset(results, from_date=None, to_date=None):
     chunk_vars = [v for v in list(data.variables) if ('chunk' in v)]
     data = data.drop_vars(chunk_vars)
 
-    stn_vars = [v for v in list(data.data_vars) if ('time' not in data[v].dims) and (v not in ['station_id', 'lon', 'lat'])]
-    data = data.drop_vars(stn_vars)
+    # if 'geometry' in data.dims:
+    #     stn_vars = [v for v in list(data.data_vars) if ('time' not in data[v].dims) and (v not in ['station_id', 'lon', 'lat'])]
+    #     data = data.drop_vars(stn_vars)
 
     if 'station_geometry' in data.dims:
         stn_vars = [d for d in data.variables if 'station_geometry' in data[d].dims]
@@ -729,9 +730,9 @@ def xr_concat(datasets: List[xr.Dataset]):
     for chunk in datasets:
         for var in chunk.data_vars:
             if isinstance(chunk[var].variable._data, np.ndarray):
-                xr3[var].loc[chunk[var].coords.indexes] = chunk[var].values
+                xr3[var].loc[chunk[var].transpose(*chunk_dict[var]['dims']).coords.indexes] = chunk[var].transpose(*chunk_dict[var]['dims']).values
             elif isinstance(chunk[var].variable._data, xr.core.indexing.MemoryCachedArray):
-                c1 = chunk[var].copy().load()
+                c1 = chunk[var].copy().load().transpose(*chunk_dict[var]['dims'])
                 xr3[var].loc[c1.coords.indexes] = c1.values
                 c1.close()
                 del c1
